/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

%{C++
#define SMARTCARDEVENT_INSERT "smartcard-insert"
#define SMARTCARDEVENT_REMOVE "smartcard-remove"
%}

[scriptable, builtinclass, uuid(cd251f11-7020-4f85-a7bd-94c98d884fa7)]
interface nsIDOMSmartCardEvent : nsIDOMEvent
{
  readonly attribute DOMString   tokenName;
  [noscript] void initSmartCardEvent(in DOMString aTypeArg,
                                     in boolean aCanBubbleArg,
                                     in boolean aCancelableArg,
                                     in DOMString aTokenNAme);
};
